import logging
import os
import unittest

from prov2bigchaindb.core import clients, local_stores
from prov2bigchaindb.core import utils
from prov2bigchaindb.tests.core import setup_test_files

log = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)


class RoleConceptTest(unittest.TestCase):
    """Test BigchainDB Role Client"""

    def setUp(self):
        self.test_prov_files = setup_test_files()
        self.host="0.0.0.0"
        self.port=9984
        if os.environ.get('BDB_HOST'):
            self.host = os.environ.get('BDB_HOST')
        if os.environ.get('BDB_PORT'):
            self.port = int(os.environ.get('BDB_PORT'))

    def tearDown(self):
        del self.host
        del self.port
        db = local_stores.SqliteStore()
        db.clean_tables()
        del db
        del self.test_prov_files

    @unittest.skip("testing skipping")
    def test_simple_prov_doc(self):
        prov_document = utils.to_prov_document(content=self.test_prov_files["simple"])
        role_client = clients.RoleConceptClient(host=self.host, port=self.port)
        tx_ids = role_client.save_document(prov_document)
        doc = role_client.get_document(tx_ids)
        self.assertEqual(len(prov_document.get_records()), len(doc.get_records()))
        self.assertEqual(prov_document, doc)

    #@unittest.skip("testing skipping")
    def test_simple2_prov_doc(self):
        prov_document = utils.to_prov_document(content=self.test_prov_files["simple2"])
        role_client = clients.RoleConceptClient(host=self.host, port=self.port)
        tx_ids = role_client.save_document(prov_document)
        doc = role_client.get_document(tx_ids)
        self.assertEqual(len(prov_document.get_records()), len(doc.get_records()))
        self.assertEqual(prov_document, doc)
        print(tx_ids)

    @unittest.skip("testing skipping")
    def test_thesis_prov_doc(self):
        prov_document = utils.to_prov_document(content=self.test_prov_files["thesis"])
        role_client = clients.RoleConceptClient(host=self.host, port=self.port)
        tx_ids = role_client.save_document(prov_document)
        doc = role_client.get_document(tx_ids)
        self.assertEqual(len(prov_document.get_records()), len(doc.get_records()))
        self.assertEqual(prov_document, doc)

    @unittest.skip("testing skipping")
    def test_quantified_prov_doc(self):
        prov_document = utils.to_prov_document(content=self.test_prov_files["quantified"])
        role_client = clients.RoleConceptClient(host=self.host, port=self.port)
        tx_ids = role_client.save_document(prov_document)
        doc = role_client.get_document(tx_ids)
        self.assertEqual(len(prov_document.get_records()), len(doc.get_records()))
        self.assertEqual(prov_document, doc)



import logging
import unittest
from unittest import mock
import sqlite3
from prov2bigchaindb.core import local_stores, exceptions

log = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)


class SqliteStoreTest(unittest.TestCase):
    def setUp(self):
        self.account_id = 'Local_Test_Account'
        self.public_key = 'public'
        self.private_key = 'private'

    def tearDown(self):
        del self.account_id
        del self.public_key
        del self.private_key

    @mock.patch('prov2bigchaindb.core.local_stores.sqlite3')
    def test_init(self, mock_sqlite3):
        local_stores.SqliteStore()
        mock_sqlite3.connect.assert_called_with(':memory:')
        mock_sqlite3.connect().execute.assert_called_with(
            '''CREATE TABLE IF NOT EXISTS accounts (account_id TEXT, public_key TEXT, private_key TEXT, tx_id TEXT, PRIMARY KEY (account_id, public_key))''')

    @mock.patch('prov2bigchaindb.core.local_stores.sqlite3')
    def test_set_account(self, mock_sqlite3):
        db = local_stores.SqliteStore()
        db.write_account(self.account_id, self.public_key, self.private_key)
        mock_sqlite3.connect().execute.assert_called_with('INSERT INTO accounts VALUES (?,?,?,?)',
                                                          (self.account_id, self.public_key, self.private_key, None))

    @mock.patch('prov2bigchaindb.core.local_stores.sqlite3')
    def test_get_account(self, mock_sqlite3):
        db = local_stores.SqliteStore()
        db.get_account(self.account_id)
        mock_sqlite3.connect().cursor().execute.assert_called_with('SELECT * FROM accounts WHERE account_id=?',
                                                                   (self.account_id,))

    def test_live_write_account(self):
        db = local_stores.SqliteStore()
        db.clean_tables()
        db.write_account(self.account_id, self.public_key, self.private_key)
        with self.assertRaises(sqlite3.IntegrityError):
            db.write_account(self.account_id, self.public_key, self.private_key)
        db.clean_tables()

    def test_live_get_account(self):
        db = local_stores.SqliteStore()
        db.clean_tables()
        db.write_account(self.account_id, self.public_key, self.private_key)
        account_id, public_key, private_key, tx_id = db.get_account(self.account_id)
        self.assertEqual(account_id, self.account_id)
        self.assertEqual(public_key, self.public_key)
        self.assertEqual(private_key, self.private_key)
        self.assertEqual(tx_id, None)
        with self.assertRaises(exceptions.NoAccountFoundException):
            ret = db.get_account('wrong_id')
            self.assertEqual(ret, None)
        db.clean_tables()



import json
import logging

import requests
import time
from bigchaindb_driver import BigchainDB
from bigchaindb_driver import exceptions as bdb_exceptions
from lxml import etree
from prov import model

from prov2bigchaindb.core import exceptions

log = logging.getLogger(__name__)
logging.basicConfig(level=logging.DEBUG)


def to_prov_document(content: str or bytes or model.ProvDocument) -> model.ProvDocument:
    """
    Takes a string, bytes or ProvDocument as argument and return a ProvDocument
    The strings or bytes can contain JSON or XML representations of PROV

    :param content: String or BufferedReader or ProvDocument
    :return: ProvDocument
    :rtype: ProvDocument
    """
    if isinstance(content, model.ProvDocument):
        return content

    if isinstance(content, str):
        content_bytes = str.encode(content)
    else:
        content_bytes = content
    try:
        if content_bytes.find(b"{") > -1:
            return model.ProvDocument.deserialize(content=content, format='json').flattened()
        if content_bytes.find(b'<?xml') > -1:
            return model.ProvDocument.deserialize(content=content, format='xml').flattened()
        elif content_bytes.find(b'document') > -1:
            return model.ProvDocument.deserialize(content=content, format='provn').flattened()
        else:
            raise exceptions.ParseException("Invalid PROV Document of type {}".format(type(content)))

    except json.decoder.JSONDecodeError:
        raise exceptions.ParseException("Invalid PROV-JSON of type {}".format(type(content)))
    except etree.XMLSyntaxError:
        raise exceptions.ParseException("Invalid PROV-XML of type {}".format(type(content)))


def wait_until_valid(tx_id: str, bdb_connection: BigchainDB):
    """
    Waits until a transaction is valid in BigchainDB

    :param tx_id: Id of transaction to wait on
    :type tx_id: str
    :param bdb_connection: Connection object for BigchainDB
    :type bdb_connection: BigchainDB
    """
    #  TODO Raise Exception on invalid
    trials = 0
    trialsmax = 100
    while trials < trialsmax:
        try:
            result = bdb_connection.transactions.status(tx_id)
            if result.get('status') == 'valid':  # others: backlog, undecided
                break
        except bdb_exceptions.NotFoundError:
            time.sleep(1)
            trials += 1
            log.debug("Transaction %s not found in BigchainDB after %s tries out of %s trials", tx_id, trials, trialsmax)
        except bdb_exceptions.TransportError as e:
            trials += 1
            log.debug("Transport Error after %s tries out of %s trials", trials, trialsmax)
            log.debug("%s",e)
    if trials == trialsmax:
        log.error("Transaction id %s not found affer %s tries", tx_id, trialsmax)
        raise exceptions.TransactionIdNotFound(tx_id)


def is_valid_tx(tx_id: str, bdb_connection: BigchainDB) -> bool:
    """
    Checks once if a transaction is valid

    :param tx_id: Id of transaction to check
    :type tx_id: str
    :param bdb_connection: Connection object for BigchainDB
    :type bdb_connection: BigchainDB
    :return: True if valid
    :rtype: bool
    """
    try:
        status = bdb_connection.transactions.status(tx_id).get('status')
    except bdb_exceptions.NotFoundError:
        log.error("Transaction id %s was not found", tx_id)
        raise exceptions.TransactionIdNotFound(tx_id)
    if status == 'valid':
        return True
    log.warning("tx %s is %s", tx_id, status)
    return False


def is_block_to_tx_valid(tx_id: str, bdb_connection: BigchainDB) -> bool:
    """
    Checks if block with transaction is valid

    :param tx_id: Id of transaction which should be included in the
    :type tx_id: str
    :param bdb_connection: Connection object for BigchainDB
    :type bdb_connection: BigchainDB
    :return: True if transactions is in block and block is valid
    :rtype: bool
    """
    api_url = bdb_connection.info()['_links']['api_v1']
    res = requests.get(api_url + 'blocks?tx_id=' + tx_id)
    block_id = res.json()
    if len(block_id) < 1 or len(block_id) > 1:
        raise exceptions.TransactionIdNotFound(tx_id)
    res = requests.get(api_url + 'statuses?block_id=' + block_id[0])
    if res.status_code != 200:
        raise exceptions.BlockIdNotFound(block_id[0])
    status = res.json()['status']
    if status == 'valid':
        return True
    return False






import logging
import unittest
from time import sleep
from unittest import mock

from bigchaindb_driver import pool as bdpool

from prov2bigchaindb.core import utils, clients
from prov2bigchaindb.tests.core import setup_test_files

log = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)


class BaseClientTest(unittest.TestCase):
    """Test BigchainDB Base Client"""

    def setUp(self):
        self.account_id = 'Base_Client_Test'
        self.public_key = 'public'
        self.private_key = 'private'
        self.host = '127.0.0.1'
        self.port = 9984

    def tearDown(self):
        del self.account_id
        del self.public_key
        del self.private_key
        del self.host
        del self.port

    @mock.patch('prov2bigchaindb.core.clients.local_stores.SqliteStore')
    @mock.patch('prov2bigchaindb.core.clients.bd.BigchainDB')
    def test_positive_init(self, mock_bdb, mock_store):
        baseclient = clients.BaseClient(self.host, self.port)
        baseclient.connection = mock_bdb
        baseclient.accountstore = mock_store
        self.assertIsInstance(baseclient, clients.BaseClient)
        # self.assertIsInstance(baseclient.accountstore, utils.LocalAccountStore)
        self.assertIsInstance(baseclient.node, str)
        self.assertEqual(baseclient.node, 'http://127.0.0.1:9984')
        # TODO Check Instance of account_db

    @unittest.skip("testing skipping")
    def test_test_transaction(self):
        raise NotImplementedError()

    @mock.patch('prov2bigchaindb.core.clients.local_stores.SqliteStore')
    @mock.patch('prov2bigchaindb.core.clients.bd.BigchainDB')
    def test_save_document(self, mock_bdb, mock_store):
        baseclient = clients.BaseClient(self.host, self.port)
        baseclient.connection = mock_bdb
        baseclient.accountstore = mock_store
        with self.assertRaises(NotImplementedError):
            baseclient.save_document('foo')


class DocumentConceptClientTest(unittest.TestCase):
    """Test BigchainDB Base Client"""

    def setUp(self):
        self.account_id = 'Document_Client_Test'
        self.public_key = 'public'
        self.private_key = 'private'
        self.host = '127.0.0.1'
        self.port = 9984
        self.test_prov_files = setup_test_files()
        self.prov_document = utils.to_prov_document(content=self.test_prov_files["simple"])

    def tearDown(self):
        del self.account_id
        del self.public_key
        del self.private_key
        del self.host
        del self.port
        del self.test_prov_files
        del self.prov_document

    @mock.patch('prov2bigchaindb.core.clients.local_stores.SqliteStore')
    @mock.patch('prov2bigchaindb.core.clients.bd.BigchainDB')
    @mock.patch('prov2bigchaindb.core.clients.accounts.DocumentConceptAccount')
    def test_positive_init(self, mock_account, mock_dbd, mock_store):
        doc_client = clients.DocumentConceptClient(self.account_id, self.host, self.port)
        self.assertIsInstance(doc_client, clients.DocumentConceptClient)
        # self.assertIsInstance(baseclient.accountstore, utils.LocalAccountStore)
        # self.assertIsInstance(baseclient.account, accounts.DocumentModelAccount)
        self.assertIsInstance(doc_client.node, str)
        self.assertEqual(doc_client.node, 'http://127.0.0.1:9984')
        # TODO Check Instance of account_db
        # TODO Check Instance of account

    @mock.patch('prov2bigchaindb.core.clients.utils.is_valid_tx')
    @mock.patch('prov2bigchaindb.core.clients.utils.is_block_to_tx_valid')
    @mock.patch('prov2bigchaindb.core.clients.local_stores.SqliteStore')
    @mock.patch('prov2bigchaindb.core.clients.bd.BigchainDB')
    @mock.patch('prov2bigchaindb.core.clients.accounts.DocumentConceptAccount')
    def test_get_document(self, mock_account, mock_bdb, mock_store, mock_test_block, mock_test_tx):
        mock_bdb.transactions.retrieve.return_value = {'id': '1', 'asset': {
            'data': {'prov': self.prov_document.serialize(format='json')}}}
        mock_test_block.return_value = True
        mock_test_tx.return_value = True
        doc_client = clients.DocumentConceptClient(self.account_id, self.host, self.port)
        doc_client.account = mock_account
        doc_client.connection_pool = bdpool.Pool([mock_bdb])
        # Test
        document = doc_client.get_document('1')
        sleep(1)
        doc_client._get_bigchain_connection().transactions.retrieve.assert_called_with('1')
        self.assertEqual(document, self.prov_document)

    @mock.patch('prov2bigchaindb.core.clients.local_stores.SqliteStore')
    @mock.patch('prov2bigchaindb.core.clients.bd.BigchainDB')
    @mock.patch('prov2bigchaindb.core.clients.accounts.DocumentConceptAccount')
    def test_save_document(self, mock_account, mock_bdb, mock_store):
        mock_account.save_asset.return_value = '1'
        doc_client = clients.DocumentConceptClient(self.account_id, self.host, self.port)
        doc_client.account = mock_account
        doc_client.connection_pool = bdpool.Pool([mock_bdb])

        tx_id = doc_client.save_document(self.prov_document)
        doc_client.account.save_asset.assert_called_with({'prov': self.prov_document.serialize(format='json')},
                                                         mock_bdb)
        self.assertIsInstance(tx_id, str)
        self.assertEqual(tx_id, '1')


class GraphConceptClientTest(unittest.TestCase):
    """Test BigchainDB Base Client"""

    def setUp(self):
        self.account_id = 'Graph_Client_Test'
        self.public_key = 'public'
        self.private_key = 'private'
        self.host = '127.0.0.1'
        self.port = 9984
        self.test_prov_files = setup_test_files()
        self.prov_document = utils.to_prov_document(content=self.test_prov_files["simple"])

    def tearDown(self):
        del self.account_id
        del self.public_key
        del self.private_key
        del self.host
        del self.port
        del self.test_prov_files
        del self.prov_document

    @unittest.skip("testing skipping")
    @mock.patch('prov2bigchaindb.core.clients.local_stores.SqliteStore')
    @mock.patch('prov2bigchaindb.core.clients.clients.bd.BigchainDB')
    @mock.patch('prov2bigchaindb.core.clients.accounts.GraphConceptAccount')
    def test_positive_init(self, mock_account, mock_dbd, mock_store):
        graph_client = clients.GraphConceptClient(self.host, self.port)
        self.assertIsInstance(graph_client, clients.GraphConceptClient)
        # self.assertIsInstance(baseclient.accountstore, utils.LocalAccountStore)
        # self.assertIsInstance(baseclient.account, accounts.DocumentModelAccount)
        self.assertIsInstance(graph_client.node, str)
        self.assertEqual(graph_client.node, 'http://127.0.0.1:9984')

    @unittest.skip("testing skipping")
    @mock.patch('prov2bigchaindb.core.clients.local_stores.SqliteStore')
    @mock.patch('prov2bigchaindb.core.clients.accounts.GraphConceptAccount')
    def test__get_prov_element_list(self, mock_account, mock_bdb):
        graph_client = clients.GraphConceptClient(self.host, self.port)
        prov_document = utils.to_prov_document(content=self.test_prov_files["simple2"])
        prov_records = prov_document.get_records()
        prov_namespaces = prov_document.get_registered_namespaces()
        element_list = clients.GraphConceptClient.calculate_account_data(prov_document)
        for element, relations, namespace in element_list:
            # print(element)
            # print("\twith: ",relations['with_id'])
            # print("\twithout: ",relations['without_id'])
            pass

    @unittest.skip("testing skipping")
    @mock.patch('prov2bigchaindb.core.clients.utils.is_valid_tx')
    @mock.patch('prov2bigchaindb.core.clients.utils.is_block_to_tx_valid')
    @mock.patch('prov2bigchaindb.core.clients.local_stores.SqliteStore')
    @mock.patch('prov2bigchaindb.core.clients.clients.bd.BigchainDB')
    @mock.patch('prov2bigchaindb.core.clients.accounts.GraphConceptAccount')
    def test_get_document(self, mock_account, mock_bdb, mock_store, mock_test_block, mock_test_tx):
        mock_bdb.transactions.retrieve.return_value = {'id': '1', 'asset': {
            'data': {'prov': self.prov_document.serialize(format='json')}}}
        mock_test_block.return_value = True
        mock_test_tx.return_value = True
        graph_client = clients.GraphConceptClient(self.host, self.port)
        graph_client.account = mock_account
        graph_client.connection = mock_bdb
        # Test
        document = graph_client.get_document(['1'])
        sleep(1)
        graph_client.connection.transactions.retrieve.assert_called_with('1')
        self.assertEqual(document, self.prov_document)

    @unittest.skip("testing skipping")
    @mock.patch('prov2bigchaindb.core.clients.local_stores.SqliteStore')
    @mock.patch('prov2bigchaindb.core.clients.clients.bd.BigchainDB')
    @mock.patch('prov2bigchaindb.core.clients.accounts.GraphConceptAccount')
    def test_save_document(self, mock_account, mock_bdb, mock_store):
        mock_account.save_asset.return_value = '1'
        graph_client = clients.GraphConceptClient(self.host, self.port)
        graph_client.account = mock_account
        graph_client.connection = mock_bdb

        tx_id = graph_client.save_document(self.prov_document)
        graph_client.account.save_asset.assert_called_with({'prov': self.prov_document.serialize(format='json')},
                                                           mock_bdb)
        self.assertIsInstance(tx_id, str)
        self.assertEqual(tx_id, '1')


class RoleConceptClientTest(unittest.TestCase):
    """Test BigchainDB Base Client"""

    def setUp(self):
        self.account_id = 'Role_Client_Test'
        self.public_key = 'public'
        self.private_key = 'private'
        self.host = '127.0.0.1'
        self.port = 9984
        self.test_prov_files = setup_test_files()
        self.prov_document = utils.to_prov_document(content=self.test_prov_files["simple"])

    def tearDown(self):
        del self.account_id
        del self.public_key
        del self.private_key
        del self.host
        del self.port
        del self.test_prov_files
        del self.prov_document

    @unittest.skip("testing skipping")
    @mock.patch('prov2bigchaindb.core.clients.local_stores.SqliteStore')
    @mock.patch('prov2bigchaindb.core.clients.clients.bd.BigchainDB')
    @mock.patch('prov2bigchaindb.core.clients.accounts.RoleConceptAccount')
    def test_positive_init(self, mock_account, mock_bdb, mock_store):
        role_client = clients.RoleConceptClient(self.host, self.port)
        self.assertIsInstance(role_client, clients.RoleConceptClient)
        # self.assertIsInstance(baseclient.accountstore, utils.LocalAccountStore)
        # self.assertIsInstance(baseclient.account, accounts.DocumentModelAccount)
        self.assertIsInstance(role_client.node, str)
        self.assertEqual(role_client.node, 'http://127.0.0.1:9984')
        # TODO Check Instance of account_db
        # TODO Check Instance of account

    @unittest.skip("testing skipping")
    @mock.patch('prov2bigchaindb.core.clients.local_stores.SqliteStore')
    @mock.patch('prov2bigchaindb.core.clients.accounts.RoleConceptAccount')
    def test__get_prov_element_list(self, mock_account, moch_bdb):
        role_clien = clients.RoleConceptClient(self.host, self.port)
        prov_document = utils.to_prov_document(content=self.test_prov_files["simple2"])
        prov_records = prov_document.get_records()
        prov_namespaces = prov_document.get_registered_namespaces()
        element_list = clients.RoleConceptClient.calculate_account_data(prov_document)
        for element, relations, namespace in element_list:
            # print(element)
            # print("\twith: ",relations['with_id'])
            # print("\twithout: ",relations['without_id'])
            pass

    @unittest.skip("testing skipping")
    @mock.patch('prov2bigchaindb.core.clients.utils.is_valid_tx')
    @mock.patch('prov2bigchaindb.core.clients.utils.is_block_to_tx_valid')
    @mock.patch('prov2bigchaindb.core.clients.local_stores.SqliteStore')
    @mock.patch('prov2bigchaindb.core.clients.clients.bd.BigchainDB')
    @mock.patch('prov2bigchaindb.core.clients.accounts.RoleConceptAccount')
    def test_get_document(self, mock_account, mock_bdb, mock_store, mock_test_block, mock_test_tx):
        mock_bdb.transactions.retrieve.return_value = {'id': '1', 'asset': {
            'data': {'prov': self.prov_document.serialize(format='json')}}}
        mock_test_block.return_value = True
        mock_test_tx.return_value = True
        role_client = clients.RoleConceptClient(self.host, self.port)
        role_client.account = mock_account
        role_client.connection = mock_bdb
        # Test
        document = role_client.get_document(['1'])
        sleep(1)
        role_client.connection.transactions.retrieve.assert_called_with('1')
        self.assertEqual(document, self.prov_document)

    @unittest.skip("testing skipping")
    @mock.patch('prov2bigchaindb.core.clients.local_stores.SqliteStore')
    @mock.patch('prov2bigchaindb.core.clients.clients.bd.BigchainDB')
    @mock.patch('prov2bigchaindb.core.clients.accounts.RoleConceptAccount')
    def test_save_document(self, mock_account, mock_bdb, mock_store):
        mock_account.save_asset.return_value = '1'
        role_client = clients.RoleConceptClient(self.host, self.port)
        role_client.account = mock_account
        role_client.connection = mock_bdb

        tx_id = role_client.save_document(self.prov_document)
        role_client.account.save_asset.assert_called_with({'prov': self.prov_document.serialize(format='json')},
                                                          mock_bdb)
        self.assertIsInstance(tx_id, str)
        self.assertEqual(tx_id, '1')



import logging

import sqlite3
from prov2bigchaindb.core import exceptions

log = logging.getLogger(__name__)
logging.basicConfig(level=logging.DEBUG)

class BaseStore(object):
    def __init__(self, db_name: str = None):
        """
        Instantiate BaseStore object

        :param db_name: Name of database
        :type db_name: str
        """
        self.db_name = db_name

    def clean_tables(self):
        """
        Delete all entries from all tables (Used for unit tests)
        """
        raise NotImplementedError("Abstract method")

    def write_account(self, account_id: str, public_key: str, private_key: str, tx_id: str = None):
        """
        Writes a new account entry in to the table accounts

        :param tx_id: Transactions id
        :type tx_id: str
        :param account_id: Id of account
        :type account_id: str
        :param public_key: Public key of account
        :type public_key: str
        :param private_key: Private key of account
        :type private_key: str
        """
        raise NotImplementedError("Abstract method")

    def get_account(self, account_id: str) -> tuple:
        """
        Returns tuple of account from data by account_id

        :param account_id: Id of account
        :type account_id: str
        :return: Tuple with account_id, public_key, private_key and tx_id
        :rtype: tuple
        """
        raise NotImplementedError("Abstract method")

    def write_tx_id(self, account_id: str, tx_id: str):
        """
        Writes tx_id for given account_id

        :param account_id: Id of account
        :type account_id: str
        :param tx_id: Transaction id, which represents the account in BigchainDB
        :type tx_id: str
        """
        raise NotImplementedError("Abstract method")


class SqliteStore(BaseStore):
    def __init__(self, db_name: str = ':memory:'):
        """
        Instantiate LocalStore object for handling the sqlite3 database which stores all accounts (PoC!)

        :param db_name: Name of local database file
        :type db_name: str
        """
        self.conn = sqlite3.connect(db_name)
        # Create table
        self.conn.execute(
            '''CREATE TABLE IF NOT EXISTS accounts (account_id TEXT, public_key TEXT, private_key TEXT, tx_id TEXT, PRIMARY KEY (account_id, public_key))''')
        super().__init__(db_name)

    def clean_tables(self):
        """
        Delete all entries from all tables (Used for unit tests)
        """
        with self.conn:
            tables = list(self.conn.execute('''SELECT name FROM sqlite_master WHERE type IS "table"'''))
            self.conn.cursor().executescript(';'.join(["DELETE FROM %s" % i for i in tables]))

    def write_account(self, account_id: str, public_key: str, private_key: str, tx_id: str = None):
        """
        Writes a new account entry in to the table accounts

        :param tx_id: Transactions id
        :type tx_id: str
        :param account_id: Id of account
        :type account_id: str
        :param public_key: Public key of account
        :type public_key: str
        :param private_key: Private key of account
        :type private_key: str
        """
        with self.conn:
            self.conn.execute('INSERT INTO accounts VALUES (?,?,?,?)', (account_id, public_key, private_key, tx_id))

    def get_account(self, account_id: str) -> tuple:
        """
        Returns tuple of account from data by account_id

        :param account_id: Id of account
        :type account_id: str
        :return: Tuple with account_id, public_key, private_key and tx_id
        :rtype: tuple
        """
        cursor = self.conn.cursor()
        cursor.execute('SELECT * FROM accounts WHERE account_id=?', (account_id,))
        ret = cursor.fetchone()
        if ret is None:
            raise exceptions.NoAccountFoundException("No account with id " + account_id)
        return ret

    def write_tx_id(self, account_id: str, tx_id: str):
        """
        Writes tx_id for given account_id

        :param account_id: Id of account
        :type account_id: str
        :param tx_id: Transaction id, which represents the account in BigchainDB
        :type tx_id: str
        """
        with self.conn:
            self.conn.execute('UPDATE accounts SET tx_id=? WHERE account_id=? ', (tx_id, account_id))
